/* ProbeDevices.c:
 *   This demo demonstrates extract information from all available devices.
 *   (c)2007-2020 Jaroslav Fojtik
 *
 *   This WILL NOT work with original Humusofts library!
 *    HudaqOpenDevice("",) with argument "" is not supported with original hudaqlib.
 */
#include <stdio.h>
#include "hudaqlib.h"


int main(void)
{
HUDAQHANDLE h;
const HudaqResourceInfo *HRI;
int i,j;
double value;
int NoAnalogIn,NoDigitalIn,NoEncoders,NoCounters;
int dev = 1;
#ifdef _MSC_VER
unsigned __int64 DMA_MemRaw;
void *Pointer;
size_t BlockSz;
#endif
	/* Open first device found of any name. */
  h = HudaqOpenDevice("",1,0);
  if(h==0)
    {printf("No HUDAQ device found\n"); return -1;}

  while(h!=0)
  {
    printf("\n================ DEVICE FOUND ======================");

    HRI = HudaqGetDeviceResources(h);
    printf("\nBus number %d, Slot number %d.",HRI->BusNumber, HRI->SlotNumber);
    printf("\nVendorID %Xh, DeviceID %Xh.",HRI->VendorID,HRI->DeviceID);

    for(i=0; i<HRI->NumMemResources; i++)
    {
      printf("\n Memory resource %d: Base:%Xh, Length:%Xh",
          i, HRI->MemResources[i].Base, HRI->MemResources[i].Length);
    }

    for(i=0; i<HRI->NumIOResources; i++)
    {
      printf("\n IO resource %d: Base:%Xh, Length:%Xh",
        i, HRI->IOResources[i].Base, HRI->IOResources[i].Length);
    }

    NoAnalogIn = HudaqGetParameter(h,0,HudaqAINUMCHANNELS);
    printf("\nAnalog channels AI:%d / AO:%d", NoAnalogIn, (int)HudaqGetParameter(h,0,HudaqAONUMCHANNELS));
    for (i=0; i<NoAnalogIn; i++)
    {
      value = HudaqAIRead(h,i);
      printf("\n  Analog channel %d, value read %fV.", i, value);
    }

    NoDigitalIn = HudaqGetParameter(h,0,HudaqDINUMCHANNELS);
    printf("\nDigital channels DI:%d / DO:%d", NoDigitalIn, (int)HudaqGetParameter(h,0,HudaqDONUMCHANNELS));
    for (i=0; i<NoDigitalIn; i++)
    {
      printf("\n  Digital input %d: %d",i,HudaqDIRead(h,i));
    }

    NoEncoders = HudaqGetParameter(h,0,HudaqEncNUMCHANNELS);
    printf("\nEncoder channels %d", NoEncoders);
    for (i=0; i<NoEncoders; i++)
    {
      printf("\n  Encoder value %d: %d",i,HudaqEncRead(h,i));
    }

    NoCounters = HudaqGetParameter(h,0,HudaqCtrNUMCHANNELS);
    printf("\nCounter channels %d", NoCounters);
    for (i=0; i<NoCounters; i++)
    {
      printf("\n  Counted value %d: %d",i,HudaqCtrRead(h,i));
    }

    printf("\nPWM channels %d", (int)HudaqGetParameter(h, 0, HudaqPwmNUMCHANNELS));
    
#ifdef _MSC_VER
    printf("\nIRQ counter: %g (%g)",
	     HudaqGetParameter(h,0,HudaqIRQ), HudaqGetParameter(h, 0, HudaqIRQ+1));

    HudaqGetDMAinfo(h,&DMA_MemRaw,&Pointer,&BlockSz);
    printf("\nDMA mem raw %lX, pointer %p, size: %u.", DMA_MemRaw,Pointer,(unsigned)BlockSz);
#endif

    HudaqCloseDevice(h);
    h = HudaqOpenDevice("",++dev,0);
  }

  //  HudaqSetParameter(h, 0, HudaqIRQ, 1);
  //i = HudaqGetParameter(h, 0, HudaqIRQ);
  printf("\n");
return 0;
}
